#include <reg167.h>
#include <stdio.h>
#include <string.h>
#include "i2c.h"

// 10 ms Timeout fuer Bus Fehler
#define Period 20000		

// Hardwareadresse des I2C EEProms
#define EEprom 0xA0

#define ulong unsigned long
#define uint unsigned int
#define uchar unsigned char

// Definition der Port-Pins fuer SCL, SDA und deren Richtungen 
sbit SCL_Direction = DP7^5;
sbit SDA_Direction = DP7^4;
sbit SCL = P7^5; 
sbit SDA = P7^4;

sbit P310 = P3^10; 	// Namenvergabe fuer Portpins	
sbit DP310 = DP3^10;// Namenvergabe fuer Richtung

/*
  HEX.   SEGMENTE     DUALKODE      HEX.
  WERT     WERT        DIGIT       DIGIT
-------------------------------------------
    0   a,b,c,d,e,f   00111111B     0x3F
    1   b,c           00000110B     0x06
    2   a,b,d,e,g     01011011B     0x5B
    3   a,b,c,d,g     01001111B     0x4F
    4   b,c,f,g       01100110B     0x66
    5   a,c,d,f,g     01101101B     0x6D
    6   a,c,d,e,g     01111101B     0x7D
    7   a,b,c,d       00001111B     0x07
    8   a,b,c,d,e,f,g 01111111B     0x7F
    9   a,b,c,f,g     01100111B     0x67
    A   a,b,c,e,f,g   01110111B     0x77
    B   c,d,e,f,g     01111100B     0x7C
    C   a,d,e,f       00111001B     0x39
    D   b,c,d,e,g     01011110B     0x5E
    E   a,d,e,f,g     01111001B     0x79
    F   a,e,f,g       01110001B     0x71
*/

unsigned char digitval[16] = { 0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07,
                                   0x7f, 0x67, 0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71 };

unsigned char ctrlbyte = 0x47;



//Prototypen
void a7seg_setOneDirect( char digit, char value, char dot );
void a7seg_setOne( char digit, char value, char dot );
void a7seg_setAllDirect( unsigned char d1, unsigned char d2, unsigned char d3, unsigned char d4 );
void a7seg_setAll( unsigned char d1, unsigned char d2, unsigned char d3, unsigned char d4 );
void a7seg_clear();
void a7seg_setIndensity( unsigned char inten );
void a7seg_getAll( unsigned char * d1, unsigned char * d2, unsigned char * d3, unsigned char * d4 );
void a7seg_getAllDirect( unsigned char * d1, unsigned char * d2, unsigned char * d3, unsigned char * d4 );




/*
unsigned char a7seg_exists()
{
   return i2c_exists( 0x70 );
}
*/


void a7seg_setOneDirect( char digit, char value, char dot )
{
   I2C_Start();                         // IC-Bus: Startbedingung
   I2C_Write( 0x70 );                   // IC-Bus: client address
   I2C_Write( 0x00 );                   // IC-Bus: address pointer 00000000B
   I2C_Write( ctrlbyte );               // IC-Bus: control byte    X1000111B
   I2C_Start();                         // IC-Bus: restart
   I2C_Write( 0x70 );                   // IC-Bus: client address
   I2C_Write( digit );                  // IC-Bus: address pointer

   if (dot==0)                          // use dot or not
      I2C_Write( value );               // set digit
   else
      I2C_Write( value | 0x80 );        // set digit with dot

   I2C_Stop();                          // IC-Bus: Stoppbedingung
}

void a7seg_setOne( char digit, char value, char dot )
{
   a7seg_setOneDirect( digit, digitval[value], dot );
}

void a7seg_setAllDirect( unsigned char d1, unsigned char d2, unsigned char d3, unsigned char d4 )
{
   I2C_Start();                         // IC-Bus: Startbedingung
   I2C_Write( 0x70 );                   // IC-Bus: client address
   I2C_Write( 0x00 );                   // IC-Bus: address pointer 00000000B
   I2C_Write( ctrlbyte );               // IC-Bus: control byte    X1000111B

   I2C_Write( d1 );
   I2C_Write( d2 );
   I2C_Write( d3 );
   I2C_Write( d4 );

   I2C_Stop();                          // IC-Bus: Stoppbedingung   
}

void a7seg_setAll( unsigned char d1, unsigned char d2, unsigned char d3, unsigned char d4 )
{
   a7seg_setAllDirect( digitval[d1], digitval[d2], digitval[d3], digitval[d4] );
}

void a7seg_clear()
{
   a7seg_setAllDirect( 0,0,0,0 );
}

// inten from 0 to 7
void a7seg_setIndensity( unsigned char inten )
{
   ctrlbyte = 0x07 | inten<<4;
}

void a7seg_getAll( unsigned char * d1, unsigned char * d2, unsigned char * d3, unsigned char * d4 )
{
   
}

void a7seg_getAllDirect( unsigned char * d1, unsigned char * d2, unsigned char * d3, unsigned char * d4 )
{
   
}





void main()
{
   int i;

//**** Initialisation Serial Port ****//
   P310=1;         	// set Port 3.10 output latch TxD 
   DP310=1;        	// set Port 3.10 direction control TxD out      
   S0TIC=0x80;     	// set transmit interrupt flag
   S0BG= 0x33;     	// set baudrate 9600 @ 16 MC CPU clock
   S0CON=0x8011;   	// set serial mode 

  printf("OK_Start\n");
  a7seg_clear();

  while(1)
  {
   for (i=0; i<8; i++)
   {
       a7seg_setIndensity( i );
       a7seg_setOne( 3, i, 0 );
       Delay(60000);
   }
  }
     
 
}
